#  FileName:  ChangeSetupPassword.ps1   
#*=============================================================================
#* Script Name:	[Change Setup Password]
#* Last Modified:	[01/22/2009]
#* Author:		Chethana Das
#* Company:	Hewlett-Packard
#*=============================================================================
#* Purpose:	A Powershell example script to modify modify the BIOS 
#*		administrator password from the HP Client Management Interface             
#*		Copyright (c) 2008-2009, Hewlett-Packard Development Company
#*=============================================================================
$Version=	"1.0.0.0"
$ScriptName = "ChangeSetupPassword.ps1"
$Copyright = "Copyright (c) 2008-2009, Hewlett-Packard Development Company"

# =============================================================================
#  FUNCTION LISTINGS
# =============================================================================
#  Function:    	Try 
#  Arguments:  	[ScriptBlock]$Command: Set of commands for which exception handling is required.  
#		[ScriptBlock]$Catch:  Set of commands to handle the exception.
#		[ScriptBlock]$Finally: Set of commands to execute after exception handling.
# =============================================================================
#  Purpose:    Function to implement C# style try-catch-finally since powershell does not have its own.
#              
# =============================================================================
Function Try
{
    param
    (
        [ScriptBlock]$Command = $(throw "The parameter -Command is required."),
        [ScriptBlock]$Catch   = { throw $_ },
        [ScriptBlock]$Finally = {}
    )
    
    & {
        $local:ErrorActionPreference = "SilentlyContinue"
        
        trap
        {
            trap
            {
                & {
                    trap { throw $_ }
                    &$Finally
                }
                
                throw $_
            }
            
            $_ | & { &$Catch }
        }
        
        &$Command
    }

    & {
        trap { throw $_ }
        &$Finally
    }
}

# =============================================================================
#  Function:    	GetPassword
#  Arguments:  	[string]$strPrompt: String indicating type(current/new) password appearing beside the password text box
#		[string]$strPwdType: String indicating if password is encrypted as kbd or unicode
#		[REF]$strEncPwd: String to pass the password back to caller
# =============================================================================
#  Purpose:    Calls HpPwdCtl to retrieve password from user in the requested encoding
#              
# =============================================================================
Function GetPassword {
	Param(	[string]$strPrompt="Provide the BIOS  Administrator password", 
		[string]$strPwdType="kbd", [REF]$strEncPwd="")

	# If there is a password set, try to invoke the password control to collect the password string.
	# If this fails, the HP CMI Password control has not been installed.  Run "hppwdctl /install"

	if ($PwdCtl -eq $null) {
		
		#Interop.HpPwdCtl.dll is a wrapper created by VS2005
		if ($strExecutingScriptDirectory.Trim() -eq "") {
			#if script path is empty try using the current path where PS is running from.
			$path = get-location 
			$fullpath = $path.Path + "\Interop.HpPwdCtl.dll"
		} else {
			$fullpath = $strExecutingScriptDirectory + "\Interop.HpPwdCtl.dll"
		}

		[Reflection.Assembly]::LoadFile($fullpath)
		$PwdCtl = new-object HpPwdCtl.PasswordEditClass
	}
		 
	if ($PwdCtl -eq $null) {
		"Please install the HPPwdCtl utility to process the setup password by running 'hppwdctl /install'"
		# Do not proceed
		return
	} else {
		# Get the password string.
		& {
			if ($strPwdType -eq "kbd") {
				$strEncPwd.Value = $pwdctl.GetPassword($strPrompt)
			}
			else {
				#GetPasswordAsUnicode(ForceKBDToUS, Prompt)
				$strEncPwd.Value = $pwdctl.GetPasswordAsUnicode($true, $strPrompt)
			} 
			if ($strEncPwd.Trim() -eq "") {
				if ($PwdType -eq "kbd") {
					$strEncPwd = "<kbd />"
				} else {
					$strEncPwd = "<utf-16 />"
				} 
			}
		} #Catch
		Trap [SystemException] {
			"Please install the HPPwdCtl utility to process the setup password by running 'hppwdctl /install'"
		}			
	}
}

# =============================================================================
#  Function:    	ChangeSetting 
#  Arguments:  	[string]$strComputer: Array of strings of computer names to set the setting values for.
# =============================================================================
#  Purpose:    Changes value of a specified BIOS setting.      (Setup Password)
#              
# =============================================================================
Function ChangeSetting {

Param ([array]$arrComputer=@("."))

foreach ($strComputer in $arrComputer)
{
	"--------------------------------------------------------------------"
	"Computer Name: " + $strComputer
	"--------------------------------------------------------------------"

	########### Password Handling ###########################
	# See if password is set on system
	$Pwd = Get-WmiObject HP_BIOSPassword -Namespace "root\HP\InstrumentedBIOS"`
	 -ComputerName $strComputer | Where {$_.Name -eq "Setup Password"}

	$strPwd = ""
	$PwdSet = $Pwd.IsSet
	$PwdSetFrom = $Pwd.PwdSetFrom  # 0=F10, so kbd; 1=OS, so unicode
	$arrPwdEncoding = $Pwd.SupportedEncoding

	$SupportsUnicodePwd = $false  	# default is does not support unicode
	$arrPwdEncoding = $Pwd.SupportedEncoding
	foreach ($enc in $arrPwdEncoding) {
		if ($enc.Trim().Length -gt 0) {
			if ($enc.Trim() -eq "utf-16") {
				$SupportsUnicodePwd = $true
			}		
		}
	}

	if ($SupportsUnicodePwd -eq $true) {
		if ($PwdSet -gt 0 -and $PwdSetFrom -eq 0) {
			$PwdType = "kbd"
		} else {
			$PwdType = "unicode"
		}
	} else {
		$PwdType = "kbd"
	}
	
	"Password is set and password format is " + $PwdType


	If ($PwdSet -gt 0) {

		# Get the current password string.
		$strPrompt = "Provide the BIOS Administrator password for computer '" + $strComputer + "'"
		$strEncPwd = ""
		GetPassword $strPrompt $PwdType ([REF]$strEncPwd)
		$strCurPwd = $strEncPwd

		if (($strCurPwd -eq $null) -or ($strCurPwd -eq "") -or ($strCurPwd.Length -eq 0) ) {
			"ERROR: The BIOS Administrator password is not provided"
			return
		}
	}

	# Get new password string.
	if ($SupportsUnicodePwd -eq $true) {
		$PwdType = "unicode"
	} else {
		$PwdType = "kbd"
	}
	$strPrompt = "Set the new BIOS Administrator password for computer '" + $strComputer + "'"
	$strEncPwd = ""
	GetPassword $strPrompt $PwdType ([REF]$strEncPwd)
	$strNewPwd = $strEncPwd

	$strPrompt = "Confirm the new BIOS Administrator password for computer '" + $strComputer + "'"
	$strEncPwd = ""
	GetPassword $strPrompt $PwdType ([REF]$strEncPwd)
	$strConfirmPwd = $strEncPwd
			
	if (($strNewPwd -ne $null) -and ($strConfirmPwd -ne $null) -and ($strNewPwd.Trim() -ne $strConfirmPwd.Trim())) {
		"ERROR: The new BIOS administrator password could not be confirmed."
		return
	}
	
	if ($strNewPwd.Trim() -eq "") {
		if ($PwdType -eq "kbd") {
			$strNewPwd = "<kbd />"
		} else {
			$strNewPwd = "<utf-16 />"
		} 
	}

	########### End Password Handling ###########################

	$Bios = Get-WmiObject HP_BIOSSettingInterface -Namespace "root\HP\InstrumentedBIOS" `
	-ComputerName $strComputer

	$ret = $Bios.SetBiosSetting("Setup Password", $strNewPwd, $strCurPwd) 
	
	switch ($ret.Return)
	{
		0 {$strReturn = "Success"}
		1 {$strReturn = "Not Supported"}
		2 {$strReturn = "Unspecified Error"}
		3 {$strReturn = "Timeout"}
		4 {$strReturn = "Failed"}
		5 {$strReturn = "Invalid Parameter"}
		6 {$strReturn = "Access Denied"}
		32768 {$strReturn = "Security Policy is violated"}
		32769 {$strReturn = "Dependency Condition is not met"}
		32770 {$strReturn = "Unsupported Configuration"}
		default {$strReturn = ""}
	}
	
	"SetBiosSetting() returned (" + $ret.Return + ") " + $strReturn
}
}

#*=============================================================================
#* SCRIPT BODY
#*=============================================================================
#Write version string
$ScriptName + " Version " + $Version + "`n" + $Copyright + "`n"

# Parse command line arguments
[bool]$run = $true
[array]$arrComputer = @(".")
foreach ($i in $args) {
	if(($i -eq "/?") -or ($i -eq "-?")) {
		"============================================="
		"Usage:`nChangeSetupPassword.ps1 <.> <computer1> <computer2>"
		"where`n<.> <computer1> ...: An optional list of computers to query specified by name or IP address, otherwise the local system is used.  The dot nomenclature can also be used to include the local computer in the computer list."
		"Example1:`nChangeSetupPassword.ps1 . remote1 `n(will let you change setup password on current computer and remote1 computer)"
		"============================================="
		$run = $false
	}
	else
	{
		# collect computer names
		if($i -ne "."){$arrComputer += $i}
	}
}
if ($run)
{
	# Get the folder of the script file.  Used to load the password interop dll
	$strExecutingScriptDirectory = Split-Path -Path $MyInvocation.MyCommand.Definition -Parent

	if($arrComputer.Count -eq 0){
		$arrComputer = @(".")
	}

	# Call the "ChangeSetting" function.
	ChangeSetting $arrComputer
}
#*=============================================================================
#* END OF SCRIPT
#*=============================================================================





# SIG # Begin signature block
# MIIWJgYJKoZIhvcNAQcCoIIWFzCCFhMCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQU+omFyCsWfjHQ11e3K1mUMC0c
# 17ygghGqMIIDejCCAmKgAwIBAgIQOCXX+vhhr570kOcmtdZa1TANBgkqhkiG9w0B
# AQUFADBTMQswCQYDVQQGEwJVUzEXMBUGA1UEChMOVmVyaVNpZ24sIEluYy4xKzAp
# BgNVBAMTIlZlcmlTaWduIFRpbWUgU3RhbXBpbmcgU2VydmljZXMgQ0EwHhcNMDcw
# NjE1MDAwMDAwWhcNMTIwNjE0MjM1OTU5WjBcMQswCQYDVQQGEwJVUzEXMBUGA1UE
# ChMOVmVyaVNpZ24sIEluYy4xNDAyBgNVBAMTK1ZlcmlTaWduIFRpbWUgU3RhbXBp
# bmcgU2VydmljZXMgU2lnbmVyIC0gRzIwgZ8wDQYJKoZIhvcNAQEBBQADgY0AMIGJ
# AoGBAMS18lIVvIiGYCkWSlsvS5Frh5HzNVRYNerRNl5iTVJRNHHCe2YdicjdKsRq
# CvY32Zh0kfaSrrC1dpbxqUpjRUcuawuSTksrjO5YSovUB+QaLPiCqljZzULzLcB1
# 3o2rx44dmmxMCJUe3tvvZ+FywknCnmA84eK+FqNjeGkUe60tAgMBAAGjgcQwgcEw
# NAYIKwYBBQUHAQEEKDAmMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC52ZXJpc2ln
# bi5jb20wDAYDVR0TAQH/BAIwADAzBgNVHR8ELDAqMCigJqAkhiJodHRwOi8vY3Js
# LnZlcmlzaWduLmNvbS90c3MtY2EuY3JsMBYGA1UdJQEB/wQMMAoGCCsGAQUFBwMI
# MA4GA1UdDwEB/wQEAwIGwDAeBgNVHREEFzAVpBMwETEPMA0GA1UEAxMGVFNBMS0y
# MA0GCSqGSIb3DQEBBQUAA4IBAQBQxUvIJIDf5A0kwt4asaECoaaCLQyDFYE3CoIO
# LLBaF2G12AX+iNvxkZGzVhpApuuSvjg5sHU2dDqYT+Q3upmJypVCHbC5x6CNV+D6
# 1WQEQjVOAdEzohfITaonx/LhhkwCOE2DeMb8U+Dr4AaH3aSWnl4MmOKlvr+ChcNg
# 4d+tKNjHpUtk2scbW72sOQjVOCKhM4sviprrvAchP0RBCQe1ZRwkvEjTRIDroc/J
# ArQUz1THFqOAXPl5Pl1yfYgXnixDospTzn099io6uE+UAKVtCoNd+V5T9BizVw9w
# w/v1rZWgDhfexBaAYMkPK26GBPHr9Hgn0QXF7jRbXrlJMvIzMIIDxDCCAy2gAwIB
# AgIQR78Zld+NUkZD99ttSA0xpDANBgkqhkiG9w0BAQUFADCBizELMAkGA1UEBhMC
# WkExFTATBgNVBAgTDFdlc3Rlcm4gQ2FwZTEUMBIGA1UEBxMLRHVyYmFudmlsbGUx
# DzANBgNVBAoTBlRoYXd0ZTEdMBsGA1UECxMUVGhhd3RlIENlcnRpZmljYXRpb24x
# HzAdBgNVBAMTFlRoYXd0ZSBUaW1lc3RhbXBpbmcgQ0EwHhcNMDMxMjA0MDAwMDAw
# WhcNMTMxMjAzMjM1OTU5WjBTMQswCQYDVQQGEwJVUzEXMBUGA1UEChMOVmVyaVNp
# Z24sIEluYy4xKzApBgNVBAMTIlZlcmlTaWduIFRpbWUgU3RhbXBpbmcgU2Vydmlj
# ZXMgQ0EwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCpyrKkzM0grwp9
# iayHdfC0TvHfwQ+/Z2G9o2Qc2rv5yjOrhDCJWH6M22vdNp4Pv9HsePJ3pn5vPL+T
# rw26aPRslMq9Ui2rSD31ttVdXxsCn/ovax6k96OaphrIAuF/TFLjDmDsQBx+uQ3e
# P8e034e9X3pqMS4DmYETqEcgzjFzDVctzXg0M5USmRK53mgvqubjwoqMKsOLIYdm
# vYNYV291vzyqJoddyhAVPJ+E6lTBCm7E/sVK3bkHEZcifNs+J9EeeOyfMcnx5iIZ
# 28SzR0OaGl+gHpDkXvXufPF9q2IBj/VNC97QIlaolc2uiHau7roN8+RN2aD7aKCu
# FDuzh8G7AgMBAAGjgdswgdgwNAYIKwYBBQUHAQEEKDAmMCQGCCsGAQUFBzABhhho
# dHRwOi8vb2NzcC52ZXJpc2lnbi5jb20wEgYDVR0TAQH/BAgwBgEB/wIBADBBBgNV
# HR8EOjA4MDagNKAyhjBodHRwOi8vY3JsLnZlcmlzaWduLmNvbS9UaGF3dGVUaW1l
# c3RhbXBpbmdDQS5jcmwwEwYDVR0lBAwwCgYIKwYBBQUHAwgwDgYDVR0PAQH/BAQD
# AgEGMCQGA1UdEQQdMBukGTAXMRUwEwYDVQQDEwxUU0EyMDQ4LTEtNTMwDQYJKoZI
# hvcNAQEFBQADgYEASmv56ljCRBwxiXmZK5a/gqwB1hxMzbCKWG7fCCmjXsjKkxPn
# BFIN70cnLwA4sOTJk06a1CJiFfc/NyFPcDGA8Ys4h7Po6JcA/s9Vlk4k0qknTnqu
# t2FB8yrO58nZXt27K4U+tZ212eFX/760xX71zwye8Jf+K9M7UhsbOCf3P0owggS/
# MIIEKKADAgECAhBBkaFaOXjfz0llZjgdTHXCMA0GCSqGSIb3DQEBBQUAMF8xCzAJ
# BgNVBAYTAlVTMRcwFQYDVQQKEw5WZXJpU2lnbiwgSW5jLjE3MDUGA1UECxMuQ2xh
# c3MgMyBQdWJsaWMgUHJpbWFyeSBDZXJ0aWZpY2F0aW9uIEF1dGhvcml0eTAeFw0w
# NDA3MTYwMDAwMDBaFw0xNDA3MTUyMzU5NTlaMIG0MQswCQYDVQQGEwJVUzEXMBUG
# A1UEChMOVmVyaVNpZ24sIEluYy4xHzAdBgNVBAsTFlZlcmlTaWduIFRydXN0IE5l
# dHdvcmsxOzA5BgNVBAsTMlRlcm1zIG9mIHVzZSBhdCBodHRwczovL3d3dy52ZXJp
# c2lnbi5jb20vcnBhIChjKTA0MS4wLAYDVQQDEyVWZXJpU2lnbiBDbGFzcyAzIENv
# ZGUgU2lnbmluZyAyMDA0IENBMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKC
# AQEAvrzuvH7vg+vgN0/7AxA4vgjSjH2d+pJ/GQzCa+5CUoze0xxIEyXqwWN6+VFl
# 7tOqO/XwlJwr+/Jm1CTa9/Wfbhk5NrzQo3YIHiInJGw4kSfihEmuG4qh/SWCLBAw
# 6HGrKOh3SlHx7M348FTUb8DjbQqP2dhkjWOyLU4n9oUO/m3jKZnihUd8LYZ/6FeP
# rWfCMzKREyD8qSMUmm3ChEt2aATVcSxdIfqIDSb9Hy2RK+cBVU3ybTUogt/Za1y2
# 1tmqgf1fzYO6Y53QIvypO0Jpso46tby0ng9exOosgoso/VMIlt21ASDR+aUY58Du
# UXA34bYFSFJIbzjqw+hse0SEuwIDAQABo4IBoDCCAZwwEgYDVR0TAQH/BAgwBgEB
# /wIBADBEBgNVHSAEPTA7MDkGC2CGSAGG+EUBBxcDMCowKAYIKwYBBQUHAgEWHGh0
# dHBzOi8vd3d3LnZlcmlzaWduLmNvbS9ycGEwMQYDVR0fBCowKDAmoCSgIoYgaHR0
# cDovL2NybC52ZXJpc2lnbi5jb20vcGNhMy5jcmwwHQYDVR0lBBYwFAYIKwYBBQUH
# AwIGCCsGAQUFBwMDMA4GA1UdDwEB/wQEAwIBBjARBglghkgBhvhCAQEEBAMCAAEw
# KQYDVR0RBCIwIKQeMBwxGjAYBgNVBAMTEUNsYXNzM0NBMjA0OC0xLTQzMB0GA1Ud
# DgQWBBQI9VHo+/49PWQ2fGjPW3io37nFNzCBgAYDVR0jBHkwd6FjpGEwXzELMAkG
# A1UEBhMCVVMxFzAVBgNVBAoTDlZlcmlTaWduLCBJbmMuMTcwNQYDVQQLEy5DbGFz
# cyAzIFB1YmxpYyBQcmltYXJ5IENlcnRpZmljYXRpb24gQXV0aG9yaXR5ghBwuuQd
# ENkpNLY4ynsDzLq/MA0GCSqGSIb3DQEBBQUAA4GBAK46F7hKe1X6ZFXsQKTtSUGQ
# mZyJvK8uHcp4I/kcGQ9/62i8MtmION7cP9OJtD+xgpbxpFq67S4m0958AW4ACgCk
# BpIRSAlA+RwYeWcjJOC71eFQrhv1Dt3gLoHNgKNsUk+RdVWKuiLy0upBdYgvY1V9
# HlRalVnK2TSBwF9e9nq1MIIFnTCCBIWgAwIBAgIQCJlFMf3xsuu4x4Ib9lD9zzAN
# BgkqhkiG9w0BAQUFADCBtDELMAkGA1UEBhMCVVMxFzAVBgNVBAoTDlZlcmlTaWdu
# LCBJbmMuMR8wHQYDVQQLExZWZXJpU2lnbiBUcnVzdCBOZXR3b3JrMTswOQYDVQQL
# EzJUZXJtcyBvZiB1c2UgYXQgaHR0cHM6Ly93d3cudmVyaXNpZ24uY29tL3JwYSAo
# YykwNDEuMCwGA1UEAxMlVmVyaVNpZ24gQ2xhc3MgMyBDb2RlIFNpZ25pbmcgMjAw
# NCBDQTAeFw0wODEyMTgwMDAwMDBaFw0xMTEyMTgyMzU5NTlaMIHcMQswCQYDVQQG
# EwJVUzETMBEGA1UECBMKQ2FsaWZvcm5pYTESMBAGA1UEBxMJUGFsbyBBbHRvMSAw
# HgYDVQQKFBdIZXdsZXR0LVBhY2thcmQgQ29tcGFueTE+MDwGA1UECxM1RGlnaXRh
# bCBJRCBDbGFzcyAzIC0gTWljcm9zb2Z0IFNvZnR3YXJlIFZhbGlkYXRpb24gdjIx
# IDAeBgNVBAsUF0hld2xldHQtUGFja2FyZCBDb21wYW55MSAwHgYDVQQDFBdIZXds
# ZXR0LVBhY2thcmQgQ29tcGFueTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoC
# ggEBALydQVhXR+j196HFpbtD3cnX0SXBJUO0ayNWcm20ww2PIgyY4BnIk+hz6hug
# enjq2U3tGSykdhSfEIfADwAznYTHmtmotb2nR1EkHdON+wkrRH+lPeP6g8uIY1BI
# vVgzMQMkpGcFyt8mvR0t7xS/00lwk8HZctVcjXk1ncmr+yR5OWnoA+IwVB7YphCV
# t2bEKnaMxDBj8QWXRpf5qWbgZjgkLlOnENGDgusItfvYpSzroM3ayESROJegPIDv
# /UhJUFLvGN+RaoA2stAatL1279PuTvh1SreI9CCxrspDTzr9NcchOmKrB8X0FOZz
# gHjn/ya2zpUQ4usNp5fSVRjmPBMCAwEAAaOCAX8wggF7MAkGA1UdEwQCMAAwDgYD
# VR0PAQH/BAQDAgeAMEAGA1UdHwQ5MDcwNaAzoDGGL2h0dHA6Ly9DU0MzLTIwMDQt
# Y3JsLnZlcmlzaWduLmNvbS9DU0MzLTIwMDQuY3JsMEQGA1UdIAQ9MDswOQYLYIZI
# AYb4RQEHFwMwKjAoBggrBgEFBQcCARYcaHR0cHM6Ly93d3cudmVyaXNpZ24uY29t
# L3JwYTATBgNVHSUEDDAKBggrBgEFBQcDAzB1BggrBgEFBQcBAQRpMGcwJAYIKwYB
# BQUHMAGGGGh0dHA6Ly9vY3NwLnZlcmlzaWduLmNvbTA/BggrBgEFBQcwAoYzaHR0
# cDovL0NTQzMtMjAwNC1haWEudmVyaXNpZ24uY29tL0NTQzMtMjAwNC1haWEuY2Vy
# MB8GA1UdIwQYMBaAFAj1Uej7/j09ZDZ8aM9beKjfucU3MBEGCWCGSAGG+EIBAQQE
# AwIEEDAWBgorBgEEAYI3AgEbBAgwBgEBAAEB/zANBgkqhkiG9w0BAQUFAAOCAQEA
# kXBGP2UhVPzkpr20FxLPXGoYy3OonVMZGh8TMguzYpMC/CcFvz8v0V7t90FnRSCE
# pMsyoAgLzjj3vE15zxOC5BXn65GsYVz2Yk3kQyD4Q42tM0KxuTg3Vtf1c7tdtRIY
# DT19KznHuUKUCmGsb41g2hF0Y3C8qahK0KX0zn6sb+Vs4Qds2zB5sIroG6KcfXin
# vJ7yWtQQY/v14vVaG0j3C0lnwIa03b9tqkPMy9DaeQnSWztinosoKipw+nKbFlMl
# gHghPHnSFfn0lHK67icWxqaUiEUylY6AQK8nSAArff1+JHLMlHkghKj+igtWLM2F
# 69ZIVEM6/JtuPpIICI6qqTGCA+YwggPiAgEBMIHJMIG0MQswCQYDVQQGEwJVUzEX
# MBUGA1UEChMOVmVyaVNpZ24sIEluYy4xHzAdBgNVBAsTFlZlcmlTaWduIFRydXN0
# IE5ldHdvcmsxOzA5BgNVBAsTMlRlcm1zIG9mIHVzZSBhdCBodHRwczovL3d3dy52
# ZXJpc2lnbi5jb20vcnBhIChjKTA0MS4wLAYDVQQDEyVWZXJpU2lnbiBDbGFzcyAz
# IENvZGUgU2lnbmluZyAyMDA0IENBAhAImUUx/fGy67jHghv2UP3PMAkGBSsOAwIa
# BQCgcDAQBgorBgEEAYI3AgEMMQIwADAZBgkqhkiG9w0BCQMxDAYKKwYBBAGCNwIB
# BDAcBgorBgEEAYI3AgELMQ4wDAYKKwYBBAGCNwIBFTAjBgkqhkiG9w0BCQQxFgQU
# 1RjM2QWzoOoGYumoJyjVMo9nwEAwDQYJKoZIhvcNAQEBBQAEggEAp18WZXLAlkDh
# x2fgvUjmGtZUQ01p8urBMUxqU1e8fHzj0UdepGZOlEmfPq9SQt98vqDfApkkpOPn
# OO+nYI1nk7UUOqMVAe/XRFrQZ+oJfhe+3iaH7yOyuSSLivdsjtk/TJe1hFKV5ag2
# rA+4kUfjvjk/NvGtmsvhvzpDIrW0I8WM58vS8mjYDNR4/slfIjW8j9NpzS5oNYH1
# dKc1kf+/uWrXHqroNB3yaEYUmj9mwNhybnAks6frsRepeCKZoDSBqLXA18Xu1F1A
# u+YkKmpZVhbKFdQ/wH6dx8bLCey9PRDUuxu3Qmah7jx8ao98bhmmAfk9mE2CMuFb
# G7gSlssK3aGCAX8wggF7BgkqhkiG9w0BCQYxggFsMIIBaAIBATBnMFMxCzAJBgNV
# BAYTAlVTMRcwFQYDVQQKEw5WZXJpU2lnbiwgSW5jLjErMCkGA1UEAxMiVmVyaVNp
# Z24gVGltZSBTdGFtcGluZyBTZXJ2aWNlcyBDQQIQOCXX+vhhr570kOcmtdZa1TAJ
# BgUrDgMCGgUAoF0wGAYJKoZIhvcNAQkDMQsGCSqGSIb3DQEHATAcBgkqhkiG9w0B
# CQUxDxcNMDkwNDIyMjIxMzA4WjAjBgkqhkiG9w0BCQQxFgQUzD/X6M2Y+MqTRSKu
# Jv0q03AcgtcwDQYJKoZIhvcNAQEBBQAEgYALe1pxcmxIGRL2pYxxIkqHdrKc7F/0
# thYGm2uSQtd/XbDIoZEK4S9G3YRrEflgGqWXSIo0CFWusl2w4CkRmnmde3Rd4Ojo
# cxu+h94J/2a08uvdhn3zfTEFz1ntqzw1HdVwB/9glSQ6W+1mVTgUi2posb/PpzYV
# v1LbLST5p3RrRA==
# SIG # End signature block
